@@ -149,6 +149,11 @@ test_that("LHS of nested `!!` is expanded (#405)", {
   expect_identical_(expr(!!1 + !!2 + foo(!!3) + !!4), quote(1 + 2 + foo(3) + 4))
 })
 
+test_that("operators with zero or one argument work (#652)", {
+  expect_identical(quo(`/`()), new_quosure(quote(`/`())))
+  expect_identical(expr(`/`(2)), quote(`/`(2)))
+})
+
 test_that("evaluates contents of `!!`", {
   expect_identical(expr(!!(1 + 2)), 3)
 })
@@ -162,14 +167,14 @@ test_that("quosures are not rewrapped", {
 })
 
 test_that("UQ() fails if called without argument", {
-  with_non_verbose_retirement({
-    quo <- quo(UQ(NULL))
-    expect_equal(quo, ~NULL)
+  scoped_lifecycle_silence()
 
-    quo <- tryCatch(quo(UQ()), error = identity)
-    expect_is(quo, "error")
-    expect_match(quo$message, "must be called with an argument")
-  })
+  quo <- quo(UQ(NULL))
+  expect_equal(quo, ~NULL)
+
+  quo <- tryCatch(quo(UQ()), error = identity)
+  expect_is(quo, "error")
+  expect_match(quo$message, "must be called with an argument")
 })
 
 
@@ -511,7 +516,7 @@ test_that("`.data[[` unquotes", {
 })
 
 test_that("it is still possible to unquote manually within `.data[[`", {
-  scoped_silent_retirement()
+  scoped_lifecycle_silence()
   foo <- "baz"
   expect_identical(expr(.data[[!!toupper(foo)]]), quote(.data[["BAZ"]]))
 })
@@ -552,13 +557,8 @@ test_that("unquoting with rlang namespace is deprecated", {
   expect_error_(dots_values(rlang::UQ(quote(.))), "`!!` in a non-quoting function")
 })
 
-test_that("UQE() is defunct", {
-  expect_error_(expr(foo$UQE(NULL)), "defunct")
-  expect_error_(UQE(), "defunct")
-})
-
 test_that("splicing language objects still works", {
-  scoped_silent_retirement()
+  scoped_lifecycle_silence()
 
   expect_identical_(exprs(!!!~foo), named_list(~foo))
   expect_identical_(exprs(!!!quote(foo(bar))), named_list(quote(foo(bar))))
@@ -574,5 +574,47 @@ test_that("splicing language objects still works", {
 })
 
 test_that("can unquote string in function position", {
-  expect_identical(expr((!!"foo")()), quote("foo"()))
+  expect_identical_(expr((!!"foo")()), quote("foo"()))
+})
+
+test_that("{{ is a quote-unquote operator", {
+  fn <- function(foo) expr(list({{ foo }}))
+  expect_identical_(fn(bar), expr(list(!!quo(bar))))
+  expect_identical_(expr(list({{ letters }})), expr(list(!!quo(!!letters))))
+  expect_error_(expr(list({{ quote(foo) }})), "must be a symbol")
+})
+
+test_that("{{ only works in quoting functions", {
+  expect_error_(
+    list2({{ "foo" }}),
+    "Can't use `{{` in a non-quoting function",
+    fixed = TRUE
+  )
+})
+
+test_that("{{ on the LHS of :=", {
+  foo <- "bar"
+  expect_identical_(exprs({{ foo }} := NA), exprs(bar = NA))
+
+  foo <- quote(bar)
+  expect_identical_(exprs({{ foo }} := NA), exprs(bar = NA))
+
+  foo <- quo(bar)
+  expect_identical_(exprs({{ foo }} := NA), exprs(bar = NA))
+
+  fn <- function(foo) exprs({{ foo }} := NA)
+  expect_identical_(fn(bar), exprs(bar = NA))
+
+  expect_error_(exprs({{ foo() }} := NA), "must be a symbol")
+})
+
+test_that("can unquote-splice in atomic capture", {
+  expect_identical_(chr("a", !!!c("b", "c"), !!!list("d")), c("a", "b", "c", "d"))
+})
+
+test_that("can unquote-splice multiple times (#771)", {
+  expect_identical(call2("foo", !!!list(1, 2), !!!list(3, 4)), quote(foo(1, 2, 3, 4)))
+  expect_identical(list2(!!!list(1, 2), !!!list(3, 4)), list(1, 2, 3, 4))
+  expect_identical(exprs(!!!list(1, 2), !!!list(3, 4)), named_list(1, 2, 3, 4))
+  expect_identical(expr(foo(!!!list(1, 2), !!!list(3, 4))), quote(foo(1, 2, 3, 4)))
 })
