@@ -485,6 +485,16 @@ test_that("global functions have `global::` prefix", {
   expect_known_trace_output(trace, file = "test-trace-global-prefix.txt")
 })
 
+test_that("local functions inheriting from global do not have `global::` prefix", {
+  skip_unless_utf8()
+
+  f <- eval_bare(expr(function(e) rlang::trace_back(e)), env(global_env()))
+  g <- function(e) f(e)
+  trace <- g(current_env())
+
+  expect_known_trace_output(trace, file = "test-trace-local-prefix.txt")
+})
+
 test_that("can trim layers of backtraces", {
   skip_unless_utf8()
 
@@ -603,7 +613,7 @@ test_that("fails when `bottom` is not on the stack", {
 })
 
 test_that("caught error does not display backtrace in knitted files", {
-  skip_if(Sys.which("pandoc") == "")
+  skip_if(!rmarkdown::pandoc_available())
 
   scoped_options(
     rlang_backtrace_on_error = NULL,
@@ -613,3 +623,21 @@ test_that("caught error does not display backtrace in knitted files", {
   error_line <- lines[[length(lines)]]
   expect_match(error_line, "foo$")
 })
+
+test_that("empty backtraces are dealt with", {
+  foo <- NULL
+
+  local({
+    env <- new.env()
+    scoped_options(rlang_trace_top_env = env)
+    tryCatch(
+      error = identity,
+      withCallingHandlers(
+        error = function(cnd) foo <<- cnd_entrace(cnd),
+        eval(quote(stop("stop")), env)
+      )
+    )
+  })
+
+  expect_identical(trace_length(foo$trace), 0L)
+})
